/**
   The eval() method takes a BTree that is an expression tree
   and evaluates it to a double value.

   The expression tree can contain the following five binary operators,
      +, -, *, /, ^
   and two unary operators
      neg, sqrt.
   The unary operators have their operand in their right subtree.
*/

public class Evaluate
{
   /**
      This eval() method is essentially a
      postorder traversal of the tree.
   */
   public static double eval(BTree tree)
   {
      double result = 0.0;

      if (tree.depth() == 0)  // a number (stops the recursion)
      {
         result = Double.parseDouble( tree.getElement() );
      }
      else if ( tree.getElement().equals("sqrt") )  // a unary operator
      {
         result = Math.sqrt( eval(tree.getRightTree()) );
      }
      else if ( tree.getElement().equals("neg") )  // a unary operator
      {
         result = -eval(tree.getRightTree());
      }
      else
      {
         double resultL = eval(tree.getLeftTree());
         double resultR = eval(tree.getRightTree());
         String op = tree.getElement();
         if ( op.equals("+") )
         {
            result = resultL + resultR;
         }
         else if ( op.equals("-") )
         {
           result = resultL - resultR;
         }
         else if ( op.equals("*") )
         {
           result = resultL * resultR;
         }
         else if ( op.equals("/") )
         {
           result = resultL / resultR;
         }
         else if ( op.equals("^") )
         {
           result = Math.pow(resultL, resultR);
         }
         else
         {
            System.out.println("Illegal operator: " + op );
         }
      }
      return result;
   }//eval()

}//Evaluate